@@ -13,13 +13,14 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 """
+from typing import Tuple
 
 import cvxpy.lin_ops.lin_utils as lu
 from cvxpy.atoms.affine.affine_atom import AffAtom
 import numpy as np
 
 
-def vstack(arg_list):
+def vstack(arg_list) -> "Vstack":
     """Wrapper on vstack to ensure list argument.
     """
     return Vstack(*arg_list)
@@ -27,12 +28,12 @@ def vstack(arg_list):
 
 class Vstack(AffAtom):
     """ Vertical concatenation """
-    def is_atom_log_log_convex(self):
+    def is_atom_log_log_convex(self) -> bool:
         """Is the atom log-log convex?
         """
         return True
 
-    def is_atom_log_log_concave(self):
+    def is_atom_log_log_concave(self) -> bool:
         """Is the atom log-log concave?
         """
         return True
@@ -54,16 +55,23 @@ class Vstack(AffAtom):
             return (rows,) + self.args[0].shape[1:]
 
     # All arguments must have the same width.
-    def validate_arguments(self):
+    def validate_arguments(self) -> None:
         model = self.args[0].shape
+        # Promote scalars.
+        if model == ():
+            model = (1,)
         for arg in self.args[1:]:
-            if len(arg.shape) != len(model) or \
-               (len(model) > 1 and model[1:] != arg.shape[1:]) or \
-               (len(model) <= 1 and model != arg.shape):
+            arg_shape = arg.shape
+            # Promote scalars.
+            if arg_shape == ():
+                arg_shape = (1,)
+            if len(arg_shape) != len(model) or \
+               (len(model) > 1 and model[1:] != arg_shape[1:]) or \
+               (len(model) <= 1 and model != arg_shape):
                 raise ValueError(("All the input dimensions except"
                                   " for axis 0 must match exactly."))
 
-    def graph_implementation(self, arg_objs, shape, data=None):
+    def graph_implementation(self, arg_objs, shape: Tuple[int, ...], data=None):
         """Stack the expressions vertically.
 
         Parameters
