@@ -14,9 +14,13 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+import cvxpy.settings as s
+from cvxpy.constraints.constraint import Constraint
 from cvxpy.atoms.atom import Atom
 import numpy as np
 from numpy import linalg as LA
+from typing import List, Tuple
+
 import scipy.sparse as sp
 
 
@@ -25,7 +29,7 @@ class log_det(Atom):
 
     """
 
-    def __init__(self, A):
+    def __init__(self, A) -> None:
         super(log_det, self).__init__(A)
 
     def numeric(self, values):
@@ -34,44 +38,46 @@ class log_det(Atom):
         For PSD matrix A, this is the sum of logs of eigenvalues of A
         and is equivalent to the nuclear norm of the matrix logarithm of A.
         """
-        sign, logdet = LA.slogdet(values[0])
+        # take symmetric part of the input.
+        symm = (values[0] + values[0].T)/2
+        sign, logdet = LA.slogdet(symm)
         if sign == 1:
             return logdet
         else:
             return -np.inf
 
     # Any argument shape is valid.
-    def validate_arguments(self):
+    def validate_arguments(self) -> None:
         X = self.args[0]
         if len(X.shape) == 1 or X.shape[0] != X.shape[1]:
             raise TypeError("The argument to log_det must be a square matrix.")
 
-    def shape_from_args(self):
+    def shape_from_args(self) -> Tuple:
         """Returns the (row, col) shape of the expression.
         """
         return tuple()
 
-    def sign_from_args(self):
+    def sign_from_args(self) -> Tuple[bool, bool]:
         """Returns sign (is positive, is negative) of the expression.
         """
         return (True, False)
 
-    def is_atom_convex(self):
+    def is_atom_convex(self) -> bool:
         """Is the atom convex?
         """
         return False
 
-    def is_atom_concave(self):
+    def is_atom_concave(self) -> bool:
         """Is the atom concave?
         """
         return True
 
-    def is_incr(self, idx):
+    def is_incr(self, idx) -> bool:
         """Is the composition non-decreasing in argument idx?
         """
         return False
 
-    def is_decr(self, idx):
+    def is_decr(self, idx) -> bool:
         """Is the composition non-increasing in argument idx?
         """
         return False
@@ -88,7 +94,7 @@ class log_det(Atom):
             A list of SciPy CSC sparse matrices or None.
         """
         X = values[0]
-        eigen_val = LA.eigvals(X)
+        eigen_val = LA.eigvalsh(X)
         if np.min(eigen_val) > 0:
             # Grad: X^{-1}.T
             D = np.linalg.inv(X).T
@@ -97,14 +103,17 @@ class log_det(Atom):
         else:
             return [None]
 
-    def _domain(self):
+    def _domain(self) -> List[Constraint]:
         """Returns constraints describing the domain of the node.
         """
         return [self.args[0] >> 0]
 
     @property
-    def value(self):
-        if not np.allclose(self.args[0].value, self.args[0].value.T.conj()):
+    def value(self) -> float:
+        if not np.allclose(self.args[0].value,
+                           self.args[0].value.T.conj(),
+                           rtol=s.ATOM_EVAL_TOL,
+                           atol=s.ATOM_EVAL_TOL):
             raise ValueError("Input matrix was not Hermitian/symmetric.")
         if any([p.value is None for p in self.parameters()]):
             return None
