@@ -48,12 +48,15 @@ def _cast_other(binary_op):
     return cast_op
 
 
+__STAR_MATMUL_COUNT__ = 1
+
 __STAR_MATMUL_WARNING__ = """
 This use of ``*`` has resulted in matrix multiplication.
 Using ``*`` for matrix multiplication has been deprecated since CVXPY 1.1.
     Use ``*`` for matrix-scalar and vector-scalar multiplication.
     Use ``@`` for matrix-matrix and matrix-vector multiplication.
     Use ``multiply`` for elementwise multiplication.
+This code path has been hit %s times so far.
 """
 
 
@@ -73,7 +76,7 @@ class Expression(u.Canonical):
     def value(self):
         """NumPy.ndarray or None : The numeric value of the expression.
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     def _value_impl(self):
         """Implementation of .value.
@@ -92,21 +95,21 @@ class Expression(u.Canonical):
             A map of variable to SciPy CSC sparse matrix; None if a variable
             value is missing.
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     @abc.abstractproperty
     def domain(self):
         """list : The constraints describing the closure of the region
            where the expression is finite.
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     def __str__(self):
         """Returns a string showing the mathematical expression.
         """
         return self.name()
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """Returns a string with information about the expression.
         """
         return "Expression(%s, %s, %s)" % (self.curvature,
@@ -117,7 +120,7 @@ class Expression(u.Canonical):
     def name(self):
         """str : The string representation of the expression.
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     @property
     def expr(self):
@@ -164,32 +167,32 @@ class Expression(u.Canonical):
         return curvature_str
 
     @perf.compute_once
-    def is_constant(self):
+    def is_constant(self) -> bool:
         """Is the expression constant?
         """
         return 0 in self.shape or all(
             arg.is_constant() for arg in self.args)
 
     @perf.compute_once
-    def is_affine(self):
+    def is_affine(self) -> bool:
         """Is the expression affine?
         """
         return self.is_constant() or (self.is_convex() and self.is_concave())
 
     @abc.abstractmethod
-    def is_convex(self):
+    def is_convex(self) -> bool:
         """Is the expression convex?
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     @abc.abstractmethod
-    def is_concave(self):
+    def is_concave(self) -> bool:
         """Is the expression concave?
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     @perf.compute_once
-    def is_dcp(self, dpp=False):
+    def is_dcp(self, dpp: bool = False) -> bool:
         """Checks whether the Expression is DCP.
 
         Parameters
@@ -208,7 +211,7 @@ class Expression(u.Canonical):
                 return self.is_convex() or self.is_concave()
         return self.is_convex() or self.is_concave()
 
-    def is_log_log_constant(self):
+    def is_log_log_constant(self) -> bool:
         """Is the expression log-log constant, ie, elementwise positive?
         """
         if not self.is_constant():
@@ -220,25 +223,25 @@ class Expression(u.Canonical):
             return self.value is not None and np.all(self.value > 0)
 
     @perf.compute_once
-    def is_log_log_affine(self):
+    def is_log_log_affine(self) -> bool:
         """Is the expression affine?
         """
         return (self.is_log_log_constant()
                 or (self.is_log_log_convex() and self.is_log_log_concave()))
 
     @abc.abstractmethod
-    def is_log_log_convex(self):
+    def is_log_log_convex(self) -> bool:
         """Is the expression log-log convex?
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     @abc.abstractmethod
-    def is_log_log_concave(self):
+    def is_log_log_concave(self) -> bool:
         """Is the expression log-log concave?
         """
-        return NotImplemented
+        raise NotImplementedError()
 
-    def is_dgp(self, dpp=False):
+    def is_dgp(self, dpp: bool = False) -> bool:
         """Checks whether the Expression is log-log DCP.
 
         Returns
@@ -252,22 +255,22 @@ class Expression(u.Canonical):
         return self.is_log_log_convex() or self.is_log_log_concave()
 
     @abc.abstractmethod
-    def is_dpp(self, context='dcp'):
+    def is_dpp(self, context='dcp') -> bool:
         """The expression is a disciplined parameterized expression.
         """
-        return NotImplemented
+        raise NotImplementedError()
 
-    def is_quasiconvex(self):
+    def is_quasiconvex(self) -> bool:
         return self.is_convex()
 
-    def is_quasiconcave(self):
+    def is_quasiconcave(self) -> bool:
         return self.is_concave()
 
-    def is_quasilinear(self):
+    def is_quasilinear(self) -> bool:
         return self.is_quasiconvex() and self.is_quasiconcave()
 
     @perf.compute_once
-    def is_dqcp(self):
+    def is_dqcp(self) -> bool:
         """Checks whether the Expression is DQCP.
 
         Returns
@@ -277,42 +280,42 @@ class Expression(u.Canonical):
         """
         return self.is_quasiconvex() or self.is_quasiconcave()
 
-    def is_hermitian(self):
+    def is_hermitian(self) -> bool:
         """Is the expression a Hermitian matrix?
         """
         return (self.is_real() and self.is_symmetric())
 
-    def is_psd(self):
+    def is_psd(self) -> bool:
         """Is the expression a positive semidefinite matrix?
         """
         # Default to False.
         return False
 
-    def is_nsd(self):
+    def is_nsd(self) -> bool:
         """Is the expression a negative semidefinite matrix?
         """
         # Default to False.
         return False
 
-    def is_quadratic(self):
+    def is_quadratic(self) -> bool:
         """Is the expression quadratic?
         """
         # Defaults to is constant.
         return self.is_constant()
 
-    def is_symmetric(self):
+    def is_symmetric(self) -> bool:
         """Is the expression symmetric?
         """
         # Defaults to false unless scalar.
         return self.is_scalar()
 
-    def is_pwl(self):
+    def is_pwl(self) -> bool:
         """Is the expression piecewise linear?
         """
         # Defaults to constant.
         return self.is_constant()
 
-    def is_qpwa(self):
+    def is_qpwa(self) -> bool:
         """Is the expression quadratic of piecewise affine?
         """
         return self.is_quadratic() or self.is_pwl()
@@ -334,45 +337,45 @@ class Expression(u.Canonical):
         return sign_str
 
     @perf.compute_once
-    def is_zero(self):
+    def is_zero(self) -> bool:
         """Is the expression all zero?
         """
         return self.is_nonneg() and self.is_nonpos()
 
     @abc.abstractmethod
-    def is_nonneg(self):
+    def is_nonneg(self) -> bool:
         """Is the expression positive?
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     @abc.abstractmethod
-    def is_nonpos(self):
+    def is_nonpos(self) -> bool:
         """Is the expression negative?
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     @abc.abstractproperty
     def shape(self):
         """tuple : The expression dimensions.
         """
-        return NotImplemented
+        raise NotImplementedError()
 
-    def is_real(self):
+    def is_real(self) -> bool:
         """Is the Leaf real valued?
         """
         return not self.is_complex()
 
     @abc.abstractproperty
-    def is_imag(self):
+    def is_imag(self) -> bool:
         """Is the Leaf imaginary?
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     @abc.abstractproperty
-    def is_complex(self):
+    def is_complex(self) -> bool:
         """Is the Leaf complex valued?
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     @property
     def size(self):
@@ -381,7 +384,7 @@ class Expression(u.Canonical):
         return np.prod(self.shape, dtype=int)
 
     @property
-    def ndim(self):
+    def ndim(self) -> int:
         """int : The number of dimensions in the expression's shape.
         """
         return len(self.shape)
@@ -391,17 +394,17 @@ class Expression(u.Canonical):
         """
         return cvxtypes.vec()(self)
 
-    def is_scalar(self):
+    def is_scalar(self) -> bool:
         """Is the expression a scalar?
         """
         return all(d == 1 for d in self.shape)
 
-    def is_vector(self):
+    def is_vector(self) -> bool:
         """Is the expression a column or row vector?
         """
         return self.ndim <= 1 or (self.ndim == 2 and min(self.shape) == 1)
 
-    def is_matrix(self):
+    def is_matrix(self) -> bool:
         """Is the expression a matrix?
         """
         return self.ndim == 2 and self.shape[0] > 1 and self.shape[1] > 1
@@ -457,14 +460,48 @@ class Expression(u.Canonical):
     def cast_to_const(expr):
         """Converts a non-Expression to a Constant.
         """
+        if isinstance(expr, list):
+            for elem in expr:
+                if isinstance(elem, Expression):
+                    raise ValueError(
+                        "The input must be a single CVXPY Expression, not a list. "
+                        "Combine Expressions using atoms such as bmat, hstack, and vstack."
+                    )
         return expr if isinstance(expr, Expression) else cvxtypes.constant()(expr)
 
+    @staticmethod
+    def broadcast(lh_expr, rh_expr):
+        """Broacast the binary operator.
+        """
+        lh_expr = Expression.cast_to_const(lh_expr)
+        rh_expr = Expression.cast_to_const(rh_expr)
+        if lh_expr.is_scalar() and not rh_expr.is_scalar():
+            lh_expr = cvxtypes.promote()(lh_expr, rh_expr.shape)
+        elif rh_expr.is_scalar() and not lh_expr.is_scalar():
+            rh_expr = cvxtypes.promote()(rh_expr, lh_expr.shape)
+        # Broadcasting.
+        if lh_expr.ndim == 2 and rh_expr.ndim == 2:
+            # Replicate dimensions of size 1.
+            dims = [max(lh_expr.shape[i], rh_expr.shape[i]) for i in range(2)]
+            # Broadcast along dim 0.
+            if lh_expr.shape[0] == 1 and lh_expr.shape[0] < dims[0]:
+                lh_expr = np.ones((dims[0], 1)) @ lh_expr
+            if rh_expr.shape[0] == 1 and rh_expr.shape[0] < dims[0]:
+                rh_expr = np.ones((dims[0], 1)) @ rh_expr
+            # Broadcast along dim 1.
+            if lh_expr.shape[1] == 1 and lh_expr.shape[1] < dims[1]:
+                lh_expr = lh_expr @ np.ones((1, dims[1]))
+            if rh_expr.shape[1] == 1 and rh_expr.shape[1] < dims[1]:
+                rh_expr = rh_expr @ np.ones((1, dims[1]))
+        return lh_expr, rh_expr
+
     @_cast_other
     def __add__(self, other):
         """Expression : Sum two expressions.
         """
         if isinstance(other, cvxtypes.constant()) and other.is_zero():
             return self
+        self, other = self.broadcast(self, other)
         return cvxtypes.add_expr()([self, other])
 
     @_cast_other
@@ -512,9 +549,13 @@ class Expression(u.Canonical):
                     warnings.warn("Forming a nonconvex expression.")
             # Because we want to discourage using ``*`` to call matmul, we
             # raise a warning to the user.
-            warnings.resetwarnings()
-            warnings.warn(__STAR_MATMUL_WARNING__, UserWarning)
-            warnings.resetwarnings()
+            with warnings.catch_warnings():
+                global __STAR_MATMUL_COUNT__
+                warnings.simplefilter("always", UserWarning, append=True)
+                msg = __STAR_MATMUL_WARNING__ % __STAR_MATMUL_COUNT__
+                warnings.warn(msg, UserWarning)
+                warnings.warn(msg, DeprecationWarning)
+                __STAR_MATMUL_COUNT__ += 1
             return cvxtypes.matmul_expr()(self, other)
 
     @_cast_other
@@ -535,9 +576,8 @@ class Expression(u.Canonical):
     def __div__(self, other):
         """Expression : One expression divided by another.
         """
+        self, other = self.broadcast(self, other)
         if (self.is_scalar() or other.is_scalar()) or other.shape == self.shape:
-            if error.warnings_enabled():
-                warnings.warn("Forming a nonconvex expression.")
             return cvxtypes.div_expr()(self, other)
         else:
             raise ValueError("Incompatible shapes for division (%s / %s)" % (
@@ -599,7 +639,7 @@ class Expression(u.Canonical):
         return PSD(self - other)
 
     # Needed for Python3:
-    def __hash__(self):
+    def __hash__(self) -> int:
         return id(self)
 
     # Comparison operators.
